//+------------------------------------------------------------------+
//|                              FLYER!      A Moving Average EA.mq4 |
//|                                          (GirlFlyer)  Marci Dunn |
//|                                  email:  girlflyer101@yahoo.com                                |
//+------------------------------------------------------------------+

/* This EA uses 4 MA's to calculate entry and exit points.  The method is based in principle upon the "Sidus" method as listed in
  the Forex Factory Forum.  I have made a few changes to the basics though.  First of all, this EA must run in the 1min chart.
  From the 1min chart, it uses the 30min timeframe to calculate the moving averages (See TimePeriod input parameter below).  
  You can also run it using a 15 minute timeframe, also.  Either timeframe seems to give decent results according to backtests.
  However, it does not work well on larger timeframes.
  
  The entry to a position is made when both WMA 5 AND WMA 8 cross the EMA 18 and EMA 28 lines.  It exits when the WMA 5 crosses back
  over the EMA 28 line for a take profit.  These entry and exit points are slightly different than those suggested by Sidus in his original
  method. If the EA is not able to take a profit when the MA's cross, then it will not exit the trade.
  At that point, the stop loss is used as the exit point, unless at a future time, it can take a profit using the same method.
  The EA does use a trailing stop feature in its execution.  If the price exceeds 60pips beyond the Open Price, it will set a 
  trailing stop/loss to a profit of +10 pips to ensure of a profit in the trade in case it reverses.  Also, this EA will adjust the 
  amount of lots per trade based upon the equity in the account.  Therefore, as the account grows, more and more lots will be opened.
  Maximum lots per trade is limited at 100 (limited by broker/server).  The size of the lot is based on a percentage of the equity which
  is user defineable.  I have set it to 8%, but you can use any percentage you feel comfortable with.  Obviously, the more you make it,
  the higher the potential growth rate, but, at the same time you risk losing your account to drawdowns.
  
  Statistics... extensive backtesting has shown that the EA achieves about 80-85% winning trades with 15-20% losses.
  It has been tested on the following pairs... EUR/USD, GBP/USD, USD/JPY, USD/CHF, USD/CAD, and GBP/JPY.
  Two values must be adjusted for what ever pair you are using.  #1... the Stoploss value, and #2, the PointAdjust value.
  These 2 values control significantly the P/L ratio.  See the input parameters section below for details on settings.
  
  Further performance....
  
  1 year test (9-01-05 to 9-29-05) for the following pairs on a $10000 starting account resulted in the following best ending balances.
  
  EUR/USD - $3,374,125  
  GBP/USD - $7,391,478  
  USD/CHF - $2,683,498
  USD/JPY - $1,724,286
  USD/CAD - $333,569     - Low profits here attributed to lack of large trends on USD/CAD.
  GBP/JPY - $1,269,975
  
  So, as you can see, the backtesting has achieved phenomenal results.
  
  Please perform your own tests, both backwards and forwards and share your results at ForexFactory.com and/or at my email:  girlflyer101@yahoo.com.
  
  Thanks for trying this EA!  - Marci Dunn (GirlFlyer)
  
  */
#property copyright "Marci Dunn"
#property link      ""

//---- input parameters


// Parameters are set for EUR/USD.  Adjust for any other pairs.

extern double  LotPercentage = 8;      // Percentage of equity to be used for each position.
extern double  MinVal=5000;            //Don't Let Equity drop below 5000 (example, can be set to whatever value you want.
extern int     OpenOrder;              // DO NOT MODIFY!
extern int     OrderFlag = 0;          // DO NOT MODIFY!
extern int     StopLossVal = 40;       //use ranges of 75-80*-100 for GBP/JPY, 35-40*-50 for USD/CHF, 45-50*-60 for GBP/USD, 40-45*-50 for EUR/USD
                                       // 30-35*-40 for USD/CAD, 35-38*-45 for USD/JPY.
                                       //The stoploss value here is key to the success rate of the EA... adjust as needed, but the values given are optimal.
                                       // Values marked by (*)asterisk are the best setting I found in the range given to achieve best profit with least Relative Drawdown %.
extern int     TimePeriod = 30;        // Key Timeframe to base buy/sell decisions on!!  Can be modified, but 30 min seems to be optimal;
extern double  PointAdjust = 0.0001;   // This factor will help filter price static out on the timeframes for volatile pairs, especially GBP/JPY.
                                       // Use 0.0 for USD/JPY, 0 for USD/CAD, and 0.0001 EUR/USD,USD/CHF, .02 for GBP/JPY
                                         // Larger values will help to lockdown entries, but will reduce overall gains by reducing number of trades.
//+------------------------------------------------------------------+
//| expert initialization function                                   |
//+------------------------------------------------------------------+
int init()
  {
//----
   
   OrderFlag = 0;

   
//----
   return(0);
  }
//+------------------------------------------------------------------+
//| expert deinitialization function                                 |
//+------------------------------------------------------------------+
int deinit()
  {
//----
   
//----
   return(0);
  }


void  CheckToOpenBuy(double  LotSize2)  // Checks to Open a Buy Position

  {
    int  x,flag,TicketNum;
    double  EMA18[5];
    double  EMA28[5];
    double  WMA5[5];  
    double  WMA8[5];  
  
    if(OrderFlag>0)
      return;
    for(x=0;x<5;x++)  // Load up the MA's into arrays.
      { 
        EMA18[x]=iMA(NULL,TimePeriod,18,0,MODE_EMA,PRICE_CLOSE,4-x); 
        EMA28[x]=iMA(NULL,TimePeriod,28,0,MODE_EMA,PRICE_CLOSE,4-x);
        WMA5[x]=iMA(NULL,TimePeriod,5,0,MODE_LWMA,PRICE_CLOSE,4-x);
        WMA8[x]=iMA(NULL,TimePeriod,8,0,MODE_LWMA,PRICE_CLOSE,4-x);
      }
    flag = 0;
    // Following checks to see if the MA'S have crossed.
    if(WMA5[4]>EMA18[4]+PointAdjust && WMA5[4]>EMA28[4]+PointAdjust && WMA8[4]>EMA18[4]+PointAdjust && WMA8[4]>EMA28[4]+PointAdjust)
      {
        for(x=0;x<4;x++)  //Check to see if the wma's have crossed the ema's recently to confirm an entry point.
          {
            if(WMA5[x]<=EMA18[x] && WMA5[x]<=EMA28[x] && WMA8[x]<=EMA18[x] && WMA8[x]<=EMA28[x])
              flag=1;  //Yes, they crossed within the last 5 periods.
          }
        if(flag==1)
          {
            TicketNum = OrderSend(Symbol(),OP_BUY,LotSize2,Ask,3,Ask-StopLossVal*Point,0,"",0,0,Green);  //Open the order.
            if(TicketNum<0)
              {
                Print("Error Opening Trade:  Error Code = ",GetLastError(),", Lotsize: ",LotSize2);
              }
            else
              {
                if(OrderSelect(TicketNum,SELECT_BY_TICKET))
                  {
                    Print("Buy Order Opened at Price:  ",OrderOpenPrice());
                    OpenOrder = TicketNum;
                    OrderFlag = 1;
                  }
              }
          }
      }
      
  }
  
void  CheckToOpenSell(double  LotSize2)  //See comments in Open Buy routine for explanations.

{
    int     x,flag,TicketNum;
    double  EMA18[5];
    double  EMA28[5];
    double  WMA5[5];  
    double  WMA8[5];  
  
    if(OrderFlag>0)
      return;
    for(x=0;x<5;x++)
      { 
        EMA18[x]=iMA(NULL,TimePeriod,18,0,MODE_EMA,PRICE_CLOSE,4-x);
        EMA28[x]=iMA(NULL,TimePeriod,28,0,MODE_EMA,PRICE_CLOSE,4-x);
        WMA5[x]=iMA(NULL,TimePeriod,5,0,MODE_LWMA,PRICE_CLOSE,4-x);
        WMA8[x]=iMA(NULL,TimePeriod,8,0,MODE_LWMA,PRICE_CLOSE,4-x);
      }
    flag = 0;
    if(WMA5[4]<EMA18[4]-PointAdjust && WMA5[4]<EMA28[4]-PointAdjust && WMA8[4]<EMA18[4]-PointAdjust && WMA8[4]<EMA28[4]-PointAdjust)
      {
        for(x=0;x<4;x++)  //Check to see if the wma's have crossed the ema's recently for an entry point.
          {
            if(WMA5[x]>=EMA18[x] && WMA5[x]>=EMA28[x] && WMA8[x]>=EMA18[x] && WMA8[x]>=EMA28[x])
              flag=1;
          }
        if(flag==1)
          {
            TicketNum = OrderSend(Symbol(),OP_SELL,LotSize2,Bid,3,Bid+StopLossVal*Point,0,"",0,0,Green);
            if(TicketNum<0)
              {
                Print("Error Opening Trade:  Error Code = ",GetLastError(),", Lotsize: ",LotSize2);
              }
            else
              {
                if(OrderSelect(TicketNum,SELECT_BY_TICKET))
                  {
                    Print("Sell Order Opened at Price:  ",OrderOpenPrice());
                    OpenOrder = TicketNum;
                    OrderFlag = 2;
                  }
              }
          }
      } 
      
  }
                  
        
      
  
void  CheckToOpen()  //Procedure to start the process to see if a trade can be opened.

  {
    double  LotSize;
   
    
    LotSize = (LotPercentage/100) * AccountEquity();
    LotSize = LotSize / 1000;
    if(LotSize>100)    //Check to see if lots have exceeded 100... good when account hits high enough equity to max out trade size.
       LotSize = 100;
    
  
    if(!OrderFlag) //If we haven't got an open order, then proceed.
      {
        if(AccountEquity()>MinVal) //if equity is lower than the amount specified by user, don't trade.
          {
            if(LotSize*1000<AccountFreeMargin())  //do we have enough margin to open our trade?  Should always be true unless errors.
              {
                CheckToOpenBuy(LotSize);
                CheckToOpenSell(LotSize);
              }
            else
              {
                Print("Not Enough Free Margin:  ",LotSize,", ",AccountFreeMargin());
              }
          }
        else
          {
            Print("Equity Value Lower than Minimum Account Value Setting");
          }
      }
  }
  
void  CheckToCloseBuy()  //Check to close any Long trades.
  {
    int  TicketNum;
    datetime  TimeOpened,Now;
    
    TicketNum = OpenOrder;
    //Check to see if WMA 5 has crossed EMA 28 for exit signal.
    if(iMA(NULL,TimePeriod,5,0,MODE_LWMA,PRICE_CLOSE,0)<iMA(NULL,TimePeriod,28,0,MODE_EMA,PRICE_CLOSE,0))
      {
        if(OrderSelect(TicketNum,SELECT_BY_TICKET))
          {
            TimeOpened = OrderOpenTime();
            Now = CurTime();
            if(Now-TimeOpened > Period()*60*2)  //if position was opened within the last 2 periods, don't close.
              {
                if(Bid>OrderOpenPrice())  // Important to prevent being whipsawed all over the place, stoploss will close losing positions.
                  {
                    if(OrderClose(TicketNum,OrderLots(),Bid,3,Red))
                      {
                        Print("Order#",TicketNum," Closed at ",OrderClosePrice());
                        OrderFlag = 0;
                      }
                  }
              }
          }
        else
          {
            Print("Error Closing Ticket#",TicketNum);
            OrderFlag = 0;
          }
      }
  }
  
void  CheckToCloseSell()  //Check to close short positions.  See CloseBuy procedure for details.
  {
    int  TicketNum;
    datetime  TimeOpened,Now;
    
    TicketNum = OpenOrder;
    if(iMA(NULL,TimePeriod,5,0,MODE_LWMA,PRICE_CLOSE,0)>iMA(NULL,TimePeriod,28,0,MODE_EMA,PRICE_CLOSE,0))
      {
        if(OrderSelect(TicketNum,SELECT_BY_TICKET))
          {
            TimeOpened = OrderOpenTime();
            Now = CurTime();
            if(Now-TimeOpened > Period()*60*2)
              {
                if(Ask<OrderOpenPrice())  // Important to prevent being whipsawed all over the place, stoploss will close losing positions.
                  {
                    if(OrderClose(TicketNum,OrderLots(),Ask,3,Red))
                      {
                        Print("Order#",TicketNum," Closed at ",OrderClosePrice());
                        OrderFlag = 0;
                      }
                  }
              }
          }
        else
          {
            Print("Error Closing Ticket#",TicketNum);
            OrderFlag = 0;
          }
      }
  }
  
void  CheckForStops()  // This checks to see if the open order was stopped out.  Needed to reset the open order flag.

  {
    datetime  X;
    
    if(OrderFlag>0)  //  Do/Did we have an open order?
      {
        if(OrderSelect(OpenOrder,SELECT_BY_TICKET))
          {
            X = OrderCloseTime();
            if(X>0)                  //Is there a close time?  if so, order was stopped out.
              {
                OrderFlag = 0;  // reset open order flag.  Allows next order to be placed.
                Print("Order# ",OpenOrder," StopLoss Close at price:  ",OrderClosePrice());
              }
          }
      }
  }
     
  
void  CheckToClose()  //Calls the procedures to close positions if necessary.

  {
    if(OrderFlag==1)
      {
        CheckToCloseBuy();
      }
    if(OrderFlag==2)
      {
        CheckToCloseSell();
      }
      
  }
  
void  CheckForModify()  // Adjust Stoploss to +10 pips profit in case profit exceeds 60 pips.

  {
    if(OrderFlag==0)
      return;
    if(OrderFlag==1)  //modify long position
      {
        if(OrderSelect(OpenOrder,SELECT_BY_TICKET))
          {
            if(Bid>OrderOpenPrice()+60*Point)
              {
                if(OrderOpenPrice()+10*Point!=OrderStopLoss())
                  {
                    //modify order for new stop loss.  Set a bogus T/P level.  Needed for the Ordermodify call.
                    if(OrderModify(OpenOrder,OrderOpenPrice(),OrderOpenPrice()+10*Point,OrderOpenPrice()+700*Point,0,Yellow)==false)
                      {
                        Print("Error Modifying Order! Code: ",GetLastError());
                      }
                  }
              }
          }
      }
    if(OrderFlag==2)  // modify short position
      {
        if(OrderSelect(OpenOrder,SELECT_BY_TICKET))
          {
            if(Ask<OrderOpenPrice()-60*Point)
              {
                if(OrderOpenPrice()-10*Point!=OrderStopLoss())
                  {
                    if(OrderModify(OpenOrder,OrderOpenPrice(),OrderOpenPrice()-10*Point,OrderOpenPrice()-700*Point,0,Yellow)==false)
                      {
                        Print("Error Modifying Order! Code: ",GetLastError());
                      }
                  }
              }
          }
      }
  }
  
//+------------------------------------------------------------------+
//| expert start function                                            |
//+------------------------------------------------------------------+
int start()
  {
//----
   
   CheckToOpen();
   CheckToClose();
   CheckForModify();
   CheckForStops();
   Print("Account Equity = ",AccountEquity()," Bid = ",Bid," Ask = ",Ask," OrderStat = ",OrderFlag);
   
//----
   return(0); 
  }
//+------------------------------------------------------------------+